﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.DirectoryServices;
using System.Web;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;

namespace RH.Net35WebConfig
{
	internal static class DotNet35Support
	{
		// ReSharper disable InconsistentNaming
		private static IList<SPWebConfigModification> _WebConfigModifications;
		private static AddRemove _AddRemove;
		// ReSharper restore InconsistentNaming

		/// <summary>
		/// add or remove web.config modifications
		/// </summary>
		/// <param name="webApplication"></param>
		/// <param name="addRemove"></param>
		internal static void AddRemoveModifications(SPWebApplication webApplication, AddRemove addRemove)
		{
			_AddRemove = addRemove;
			_WebConfigModifications = new List<SPWebConfigModification>();
			var mods = new WebConfigModification(webApplication, ".NET 3.5 Ajax Support");
			if (addRemove == AddRemove.Remove)
			{
				RemoveModifications(webApplication, mods);
				return;
			}

			var mod = mods.Create("sectionGroup[@name='system.web.extensions']", "configuration/configSections", "<sectionGroup name=\"system.web.extensions\" type=\"System.Web.Configuration.SystemWebExtensionsSectionGroup, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\"></sectionGroup>");
			ModifyConfiguration(mod);

			mod = mods.Create("sectionGroup[@name='scripting']", "configuration/configSections/sectionGroup[@name='system.web.extensions']", "<sectionGroup name=\"scripting\" type=\"System.Web.Configuration.ScriptingSectionGroup, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\"></sectionGroup>");
			ModifyConfiguration(mod);

			mod = mods.Create("section[@name='scriptResourceHandler']", "configuration/configSections/sectionGroup[@name='system.web.extensions']/sectionGroup[@name='scripting']", "<section name=\"scriptResourceHandler\" type=\"System.Web.Configuration.ScriptingScriptResourceHandlerSection, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\" requirePermission=\"false\" allowDefinition=\"MachineToApplication\"/>");
			ModifyConfiguration(mod);
			mod = mods.Create("sectionGroup[@name='webServices']", "configuration/configSections/sectionGroup[@name='system.web.extensions']/sectionGroup[@name='scripting']", "<sectionGroup name=\"webServices\" type=\"System.Web.Configuration.ScriptingWebServicesSectionGroup, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\"></sectionGroup>");
			ModifyConfiguration(mod);

			mod = mods.Create("section[@name='jsonSerialization']", "configuration/configSections/sectionGroup[@name='system.web.extensions']/sectionGroup[@name='scripting']/sectionGroup[@name='webServices']", "<section name=\"jsonSerialization\" type=\"System.Web.Configuration.ScriptingJsonSerializationSection, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\" requirePermission=\"false\" allowDefinition=\"Everywhere\" />");
			ModifyConfiguration(mod);
			mod = mods.Create("section[@name='profileService']", "configuration/configSections/sectionGroup[@name='system.web.extensions']/sectionGroup[@name='scripting']/sectionGroup[@name='webServices']", "<section name=\"profileService\" type=\"System.Web.Configuration.ScriptingProfileServiceSection, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\" requirePermission=\"false\" allowDefinition=\"MachineToApplication\" />");
			ModifyConfiguration(mod);
			mod = mods.Create("section[@name='authenticationService']", "configuration/configSections/sectionGroup[@name='system.web.extensions']/sectionGroup[@name='scripting']/sectionGroup[@name='webServices']", "<section name=\"authenticationService\" type=\"System.Web.Configuration.ScriptingAuthenticationServiceSection, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\" requirePermission=\"false\" allowDefinition=\"MachineToApplication\" />");
			ModifyConfiguration(mod);
			mods.Add(_WebConfigModifications);

            mods.AddSection("pages", "configuration/system.web");
			mods.AddSection("controls", "configuration/system.web/pages");
			mods.Save();

			mod = mods.Create("add[@tagPrefix='asp']", "configuration/system.web/pages/controls", "<add tagPrefix=\"asp\" namespace=\"System.Web.UI\" assembly=\"System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\"/>");
			ModifyConfiguration(mod);

			mod = mods.Create("add[@assembly='System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35']", "configuration/system.web/compilation/assemblies", "<add assembly=\"System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\"/>");
			ModifyConfiguration(mod);

			mod = mods.Create("add[@path='*.asmx']", "configuration/system.web/httpHandlers", "<add verb=\"*\" path=\"*.asmx\" validate=\"false\" type=\"System.Web.Script.Services.ScriptHandlerFactory, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\"/>");
			ModifyConfiguration(mod);
			mod = mods.Create("add[@path='*_AppService.axd']", "configuration/system.web/httpHandlers", "<add verb=\"*\" path=\"*_AppService.axd\" validate=\"false\" type=\"System.Web.Script.Services.ScriptHandlerFactory, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\"/>");
			ModifyConfiguration(mod);
			mod = mods.Create("add[@path='ScriptResource.axd']", "configuration/system.web/httpHandlers", "<add verb=\"GET,HEAD\" path=\"ScriptResource.axd\" type=\"System.Web.Handlers.ScriptResourceHandler, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\" validate=\"false\"/>");
			ModifyConfiguration(mod);

			mod = mods.Create("add[@name='ScriptModule']", "configuration/system.web/httpModules", "<add name=\"ScriptModule\" type=\"System.Web.Handlers.ScriptModule, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\"/>");
			ModifyConfiguration(mod);

			mod = mods.Create(
				"SafeControl[@Assembly='System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35'][@Namespace='System.Web.UI'][@TypeName='*'][@Safe='True']",
				 "configuration/SharePoint/SafeControls",
				 "<SafeControl Assembly='System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35' Namespace='System.Web.UI' TypeName='*' Safe='True' />");
			ModifyConfiguration(mod);
			mods.Add(_WebConfigModifications);

			mods.AddSection("system.web.extensions", "configuration");
			mods.AddSection("scripting", "configuration/system.web.extensions");
			mods.AddSection("webServices", "configuration/system.web.extensions/scripting");
			mods.AddSection("system.webServer", "configuration");
			mods.AddSection("modules", "configuration/system.webServer");
			mods.AddSection("handlers", "configuration/system.webServer");
			mods.Save();

			mod = mods.Create("validation[@validateIntegratedModeConfiguration='false']", "configuration/system.webServer", "<validation validateIntegratedModeConfiguration=\"false\"/>");
			ModifyConfiguration(mod);
			mod = mods.Create("add[@name='ScriptModule']", "configuration/system.webServer/modules", "<add name=\"ScriptModule\" preCondition=\"integratedMode\" type=\"System.Web.Handlers.ScriptModule, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\"/>");
			ModifyConfiguration(mod);
			mod = mods.Create("remove[@name='WebServiceHandlerFactory-Integrated']", "configuration/system.webServer/handlers", "<remove name=\"WebServiceHandlerFactory-Integrated\" />");
			ModifyConfiguration(mod);
			mod = mods.Create("add[@name='ScriptHandlerFactory']", "configuration/system.webServer/handlers", "<add name=\"ScriptHandlerFactory\" verb=\"*\" path=\"*.asmx\" preCondition=\"integratedMode\" type=\"System.Web.Script.Services.ScriptHandlerFactory, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\"/>");
			ModifyConfiguration(mod);
			mod = mods.Create("add[@name='ScriptHandlerFactoryAppServices']", "configuration/system.webServer/handlers", "<add name=\"ScriptHandlerFactoryAppServices\" verb=\"*\" path=\"*_AppService.axd\" preCondition=\"integratedMode\" type=\"System.Web.Script.Services.ScriptHandlerFactory, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\"/>");
			ModifyConfiguration(mod);
			mod = mods.Create("add[@name='ScriptResource']", "configuration/system.webServer/handlers", "<add name=\"ScriptResource\" preCondition=\"integratedMode\" verb=\"GET,HEAD\" path=\"ScriptResource.axd\" type=\"System.Web.Handlers.ScriptResourceHandler, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\" />");
			ModifyConfiguration(mod);
			mods.Add(_WebConfigModifications);

			mods.Save();
		}

		internal static void AddRemoveModificationsForSilverlight(SPWebApplication webApplication, AddRemove addRemove)
		{
			_WebConfigModifications = new List<SPWebConfigModification>();
			_AddRemove = addRemove;

			// add the xap mime type if needed
			double serverVersion = GetIISVersion();
			if (serverVersion < 7.0)
			{
				// changes are only need until IIS Version 6
				foreach (KeyValuePair<SPUrlZone, SPIisSettings> pair in webApplication.IisSettings)
				{
					// modify iis settings for each virual iis for the webapplication
					string path = string.Format("IIS://localhost/W3SVC/{0}/Root", pair.Value.PreferredInstanceId);
					Logging.WriteULSVerbose("IIS path: " + path);
					SetMimeTypeProperty(path, ".xap", "application/x-silverlight-app", addRemove);
				}
			}

			var mods = new WebConfigModification(webApplication, "RH Silverlight Support");
			if (addRemove == AddRemove.Remove)
			{
				RemoveModifications(webApplication, mods);
				return;
			}

			var mod = mods.Create("add[@assembly='System.Web.Silverlight, Version=2.0.5.0, Culture=neutral,PublicKeyToken=31bf3856ad364e35']", "configuration/system.web/compilation/assemblies", "<add assembly=\"System.Web.Silverlight, Version=2.0.5.0, Culture=neutral,PublicKeyToken=31bf3856ad364e35\" />");
			ModifyConfiguration(mod);
			mods.Add(_WebConfigModifications);
			mods.Save();
		}

		/// <summary>
		/// Detect IIS version from server variable
		/// </summary>
		/// <returns></returns>
		private static double GetIISVersion()
		{
			double version = 0.0;
			try
			{
				string serverSoftware = HttpContext.Current.Request.ServerVariables["SERVER_SOFTWARE"];

				if (serverSoftware.StartsWith("Microsoft-IIS/"))
				{
					serverSoftware = serverSoftware.Replace("Microsoft-IIS/", "");
					double.TryParse(serverSoftware, out version);
				}
			}
			catch (Exception ex)
			{
				throw new SPException("Cannot get IIS Version. " + ex.Message);
			}
			return version;
		}

		private static void RemoveModifications(SPWebApplication webApplication, WebConfigModification modification)
		{
			Collection<SPWebConfigModification> webConfigModifications = webApplication.WebConfigModifications;
			int existingModifications = webConfigModifications.Count;
			bool needsUpdate = false;

			for (int i = existingModifications - 1; i >= 0; i--)
			{
				if (webConfigModifications[i].Owner == modification.Owner)
				{
					Logging.WriteULSVerbose(string.Format("Removing modification: {0}", webConfigModifications[i].Name));
					webConfigModifications.RemoveAt(i);
					needsUpdate = true;
				}
			}

			if (needsUpdate)
			{
				Logging.WriteULSVerbose("Applying web.config modifications...");
				webApplication.Farm.Services.GetValue<SPWebService>().ApplyWebConfigModifications();
				webApplication.Update();
			}
		}

		/// <summary>
		/// add or remove the modification to the modification list
		/// </summary>
		/// <param name="modification"></param>
		private static void ModifyConfiguration(SPWebConfigModification modification)
		{
			if (_AddRemove == AddRemove.Add)
			{
				Logging.WriteULSVerbose(string.Format("Adding web.config modification: {0}", modification.Name));
				_WebConfigModifications.Add(modification);
			}
			else
			{
				Logging.WriteULSVerbose(string.Format("Removing modification: {0}", modification.Name));
				_WebConfigModifications.Remove(modification);
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="metabasePath">IIS://localhost/W3SVC/1/Root</param>
		/// <param name="newExtension">.hlp</param>
		/// <param name="newMimeType">application/winhlp</param>
		/// <param name="addRemove"></param>
		static void SetMimeTypeProperty(string metabasePath, string newExtension, string newMimeType, AddRemove addRemove)
		{
			Logging.WriteULSVerbose(string.Format("Adding {1}->{2} to the MimeMap property at {0}:", metabasePath, newExtension, newMimeType));
			try
			{
				using (var path = new DirectoryEntry(metabasePath))
				{
					PropertyValueCollection propValues = path.Properties["MimeMap"];
					Logging.WriteULSVerbose(string.Format("Old value of MimeMap has {0} elements", propValues.Count));

					object exists = null;
					foreach (object value in propValues)
					{
						// IISOle requires a reference to the Active DS IIS Namespace Provider in Visual Studio .NET
						var mimetypeObj = (IISOle.IISMimeType) value;
						Logging.WriteULSVerbose(string.Format("{0}->{1}", mimetypeObj.Extension, mimetypeObj.MimeType));
						if (newExtension == mimetypeObj.Extension) exists = value;
					}

					if (exists != null && addRemove == AddRemove.Remove)
					{
						propValues.Remove(exists);
					}

					if (exists == null && addRemove == AddRemove.Add)
					{
						var newObj = new IISOle.MimeMapClass {Extension = newExtension, MimeType = newMimeType};
						propValues.Add(newObj);
					}
					path.CommitChanges();
				}
			}
			catch (Exception ex)
			{
				if ("HRESULT 0x80005006" == ex.Message)
					Logging.WriteULSError(string.Format("Property MimeMap does not exist at {0}", metabasePath));
				else
					Logging.WriteULSError(string.Format("Failed in SetMimeTypeProperty with the following exception: \n{0}", ex.Message));
			}
		}
	}
}
