using System;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;

namespace RH.Net35WebConfig
{
	class Activation : SPFeatureReceiver
	{
		public override void FeatureActivated(SPFeatureReceiverProperties properties)
		{
			try
			{
				if (properties == null || properties.Feature == null || properties.Feature.Parent == null) return;

				var webApplication = properties.Feature.Parent as SPWebApplication;
				if (webApplication == null) return;

				Logging.WriteULSVerbose("FeatureActivated: starting with web.config modification");
				DotNet35Support.AddRemoveModifications(webApplication, AddRemove.Add);
			}
			catch (Exception ex)
			{
				string message = string.Format("Error in FeatureActivated: {0}", ex);
				Logging.WriteULSError(message);
				throw new SPException(message);
			}
		}

		public override void FeatureDeactivating(SPFeatureReceiverProperties properties)
		{
			try
			{
				if (properties == null || properties.Feature == null || properties.Feature.Parent == null) return;

				var webApplication = properties.Feature.Parent as SPWebApplication;
				if (webApplication == null) return;

				Logging.WriteULSVerbose("FeatureDeactivating: starting with web.config modification");
				DotNet35Support.AddRemoveModifications(webApplication, AddRemove.Remove);
			}
			catch (Exception ex)
			{
				string message = string.Format("Error in FeatureDeactivating: {0}", ex);
				Logging.WriteULSError(message);
				throw new SPException(message);
			}
		}

		public override void FeatureInstalled(SPFeatureReceiverProperties properties)
		{
		}

		public override void FeatureUninstalling(SPFeatureReceiverProperties properties)
		{
		}
	}
}