using System;
using System.Reflection;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;

namespace RH.Net35WebConfig
{
	class SilverlightActivation : SPFeatureReceiver
	{
		public override void FeatureActivated(SPFeatureReceiverProperties properties)
		{
			try
			{
				if (properties == null || properties.Feature == null || properties.Feature.Parent == null) return;

				var webApplication = properties.Feature.Parent as SPWebApplication;
				if (webApplication == null) return;

				if (!IsSilverlightAssemblyInGAC())
				{
					throw new SPException("Please make sure you have the \"System.Web.Silverlight\" assembly in your Global Assembly Cache. You can get the assembly by downloading the Silverlight SDK");
				}

				Logging.WriteULSVerbose("FeatureActivated: starting with web.config modification");
				DotNet35Support.AddRemoveModificationsForSilverlight(webApplication, AddRemove.Add);
			}
			catch (Exception ex)
			{
				string message = string.Format("Error in FeatureActivated: {0}", ex);
				Logging.WriteULSError(message);
				throw new SPException(message);
			}
		}

		public override void FeatureDeactivating(SPFeatureReceiverProperties properties)
		{
			try
			{
				if (properties == null || properties.Feature == null || properties.Feature.Parent == null) return;

				var webApplication = properties.Feature.Parent as SPWebApplication;
				if (webApplication == null) return;

				Logging.WriteULSVerbose("FeatureDeactivating: starting with web.config modification");
				DotNet35Support.AddRemoveModificationsForSilverlight(webApplication, AddRemove.Remove);
			}
			catch (Exception ex)
			{
				string message = string.Format("Error in FeatureDeactivating: {0}", ex);
				Logging.WriteULSError(message);
				throw new SPException(message);
			}
		}

		public override void FeatureInstalled(SPFeatureReceiverProperties properties)
		{
		}

		public override void FeatureUninstalling(SPFeatureReceiverProperties properties)
		{
		}

		private static bool IsSilverlightAssemblyInGAC()
		{
			try
			{
#pragma warning disable 612,618
				Assembly assembly = Assembly.LoadWithPartialName("System.Web.Silverlight");
#pragma warning restore 612,618
				Logging.WriteULSVerbose("Found System.Web.Silverlight: " + assembly.FullName);
				return true;
			}
			catch (Exception)
			{
				Logging.WriteULSVerbose("System.Web.Silverlight not found");
				return false;
			}
		}
	}
}